const char* default_vs =
    "#version 330 core\n"\
    "layout (location = 0) in vec3 aPos;\n"\
    "out vec2 TexCoord;\n"\
    "void main() {\n"\
    "    gl_Position = vec4(aPos, 1.0);\n"\
    "    TexCoord = aPos.xy * 0.5 + 0.5;\n"\
    "}\n"\
    "";

const char* flipped_vs =
    "#version 330 core\n"\
    "layout (location = 0) in vec3 aPos;\n"\
    "out vec2 TexCoord;\n"\
    "void main() {\n"\
    "    gl_Position = vec4(aPos, 1.0);\n"\
    "    TexCoord = vec2(aPos.x, 2.0 - aPos.y) * 0.5 + 0.5;\n"\
    "}\n"\
    "";

const char* default_fs =
    "#version 330 core\n"\
    "in vec2 TexCoord;\n"\
    "out vec4 FragColor;\n"\
    "uniform sampler2D image;\n"\
    "void main() {\n"\
    "    FragColor = texture(image, TexCoord);\n"\
    "}\n"\
    "";

const char* exposure_fs =
    "#version 330 core\n"\
    "in vec2 TexCoord;\n"\
    "out vec4 FragColor;\n"\
    "uniform sampler2D image;\n"\
    "uniform float value;\n"\
    "void main() {\n"\
    "    vec4 texColor = texture(image, TexCoord);\n"\
    "    FragColor = texColor * (value+1);\n"\
    "}\n"\
    "";

const char* brightness_fs =
    "#version 330 core\n"\
    "in vec2 TexCoord;\n"\
    "out vec4 FragColor;\n"\
    "uniform sampler2D image;\n"\
    "uniform float value;\n"\
    "void main() {\n"\
    "    vec4 texColor = texture(image, TexCoord);\n"\
    "    FragColor = texColor + value;\n"\
    "}\n"\
    "";

const char* contrast_fs =
    "#version 330 core\n"\
    "in vec2 TexCoord;\n"\
    "out vec4 FragColor;\n"\
    "uniform sampler2D image;\n"\
    "uniform float value;\n"\
    "void main() {\n"\
    "    vec4 texColor = texture(image, TexCoord);\n"\
    "    FragColor = (value+1) * (texColor-1) + 1;\n"\
    "}\n"\
    "";

const char* tint_fs =
    "#version 330 core\n"\
    "in vec2 TexCoord;\n"\
    "out vec4 FragColor;\n"\
    "uniform sampler2D image;\n"\
    "uniform float value;\n"\
    "void main() {\n"\
    "   vec4 texColor = texture(image, TexCoord);\n"\
    "   texColor.r += value;\n"\
    "   texColor.g -= value;\n"\
    "   texColor.b += value;\n"\
    "   FragColor = texColor;\n"\
    "}\n"\
    "";

const char* saturation_fs =
    "#version 330 core\n"\
    "in vec2 TexCoord;\n"\
    "out vec4 FragColor;\n"\
    "uniform sampler2D image;\n"\
    "uniform float value;\n"\
    "\n"\
    "vec4 rgb_to_hsl(vec4 rgb) {\n"\
    "   vec4 hsl = rgb;\n"\
    "\n"\
    "   float max_val = max(rgb.r, max(rgb.g, rgb.b));\n"\
    "   float min_val = min(rgb.r, min(rgb.g, rgb.b));\n"\
    "   float delta = max_val - min_val;\n"\
    "\n"\
    "   if (delta == 0) {\n"\
    "      return vec4(0,0,rgb.r,rgb.a);\n"\
    "   }\n"\
    "   \n"\
    "   if (max_val == rgb.r) {\n"\
    "      hsl.x = 60 * mod(((rgb.g-rgb.b)/delta), 6);\n"\
    "   }\n"\
    "   if (max_val == rgb.g) {\n"\
    "      hsl.x = 60 * ((rgb.b-rgb.r)/delta + 2);\n"\
    "   }\n"\
    "   if (max_val == rgb.b) {\n"\
    "      hsl.x = 60 * ((rgb.r-rgb.g)/delta + 4);\n"\
    "   }\n"\
    "\n"\
    "   if (hsl.x < 0) {\n"\
    "      hsl.x = 360+hsl.x;\n"\
    "   }\n"\
    "\n"\
    "   hsl.z = (max_val + min_val) / 2;\n"\
    "   hsl.y = delta / (1 - abs(2*(hsl.z) - 1));\n"\
    "\n"\
    "   return hsl;\n"\
    "}\n"\
    "\n"\
    "vec4 hsl_to_rgb(vec4 hsl) {\n"\
    "   float c = hsl.y * (1 - abs(2*hsl.z-1));\n"\
    "   float x = c * (1 - abs(mod(hsl.x/60, 2) - 1));\n"\
    "   float m = hsl.z - c/2;\n"\
    "\n"\
    "   float r = 0;\n"\
    "   float g = 0;\n"\
    "   float b = 0;\n"\
    "\n"\
    "   if (hsl.x >= 0 && hsl.x < 60) {\n"\
    "      r = c;\n"\
    "      g = x;\n"\
    "   }\n"\
    "   if (hsl.x >= 60 && hsl.x < 120) {\n"\
    "      r = x;\n"\
    "      g = c;\n"\
    "   }\n"\
    "   if (hsl.x >= 120 && hsl.x < 180) {\n"\
    "      g = c;\n"\
    "      b = x;\n"\
    "   }\n"\
    "   if (hsl.x >= 180 && hsl.x < 240) {\n"\
    "      g = x;\n"\
    "      b = c;\n"\
    "   }\n"\
    "   if (hsl.x >= 240 && hsl.x < 300) {\n"\
    "      r = x;\n"\
    "      b = c;\n"\
    "   }\n"\
    "   if (hsl.x >= 300 && hsl.x < 360) {\n"\
    "      r = c;\n"\
    "      b = x;\n"\
    "   }\n"\
    "\n"\
    "   return vec4(r+m, g+m, b+m, hsl.w);\n"\
    "}\n"\
    "\n"\
    "void main() {\n"\
    "   vec4 texColor = texture(image, TexCoord);\n"\
    "   \n"\
    "   vec4 texHSL = rgb_to_hsl(texColor);\n"\
    "   \n"\
    "   float modifier = texHSL.y * value;\n"\
    "\n"\
    "   if (texHSL.y + modifier > 1) {\n"\
    "      texHSL.y = 1;\n"\
    "   } else if (texHSL.y + modifier < 0) {\n"\
    "      texHSL.y = 0;\n"\
    "   } else {\n"\
    "      texHSL.y += modifier;\n"\
    "   }\n"\
    "\n"\
    "   FragColor = hsl_to_rgb(texHSL);\n"\
    "}\n"\
    "";

const char* highlights_fs =
    "#version 330 core\n"\
    "in vec2 TexCoord;\n"\
    "out vec4 FragColor;\n"\
    "uniform sampler2D image;\n"\
    "uniform float value;\n"\
    "\n"\
    "vec4 rgb_to_hsl(vec4 rgb) {\n"\
    "   vec4 hsl = rgb;\n"\
    "\n"\
    "   float max_val = max(rgb.r, max(rgb.g, rgb.b));\n"\
    "   float min_val = min(rgb.r, min(rgb.g, rgb.b));\n"\
    "   float delta = max_val - min_val;\n"\
    "\n"\
    "   if (delta == 0) {\n"\
    "      return vec4(0,0,rgb.r,rgb.a);\n"\
    "   }\n"\
    "   \n"\
    "   if (max_val == rgb.r) {\n"\
    "      hsl.x = 60 * mod(((rgb.g-rgb.b)/delta), 6);\n"\
    "   }\n"\
    "   if (max_val == rgb.g) {\n"\
    "      hsl.x = 60 * ((rgb.b-rgb.r)/delta + 2);\n"\
    "   }\n"\
    "   if (max_val == rgb.b) {\n"\
    "      hsl.x = 60 * ((rgb.r-rgb.g)/delta + 4);\n"\
    "   }\n"\
    "\n"\
    "   if (hsl.x < 0) {\n"\
    "      hsl.x = 360+hsl.x;\n"\
    "   }\n"\
    "\n"\
    "   hsl.z = (max_val + min_val) / 2;\n"\
    "   hsl.y = delta / (1 - abs(2*(hsl.z) - 1));\n"\
    "\n"\
    "   return hsl;\n"\
    "}\n"\
    "\n"\
    "vec4 hsl_to_rgb(vec4 hsl) {\n"\
    "   float c = hsl.y * (1 - abs(2*hsl.z-1));\n"\
    "   float x = c * (1 - abs(mod(hsl.x/60, 2) - 1));\n"\
    "   float m = hsl.z - c/2;\n"\
    "\n"\
    "   float r = 0;\n"\
    "   float g = 0;\n"\
    "   float b = 0;\n"\
    "\n"\
    "   if (hsl.x >= 0 && hsl.x < 60) {\n"\
    "      r = c;\n"\
    "      g = x;\n"\
    "   }\n"\
    "   if (hsl.x >= 60 && hsl.x < 120) {\n"\
    "      r = x;\n"\
    "      g = c;\n"\
    "   }\n"\
    "   if (hsl.x >= 120 && hsl.x < 180) {\n"\
    "      g = c;\n"\
    "      b = x;\n"\
    "   }\n"\
    "   if (hsl.x >= 180 && hsl.x < 240) {\n"\
    "      g = x;\n"\
    "      b = c;\n"\
    "   }\n"\
    "   if (hsl.x >= 240 && hsl.x < 300) {\n"\
    "      r = x;\n"\
    "      b = c;\n"\
    "   }\n"\
    "   if (hsl.x >= 300 && hsl.x < 360) {\n"\
    "      r = c;\n"\
    "      b = x;\n"\
    "   }\n"\
    "\n"\
    "   return vec4(r+m, g+m, b+m, hsl.w);\n"\
    "}\n"\
    "\n"\
    "void main() {\n"\
    "   vec4 texColor = texture(image, TexCoord);\n"\
    "   \n"\
    "   vec4 texHSL = rgb_to_hsl(texColor);\n"\
    "\n"\
    "   /*float coefficient = (1 - texHSL.z * 2.5) * -1;\n"\
    "   if (coefficient < 0) {\n"\
    "      coefficient = 0;\n"\
    "   }\n"\
    "\n"\
    "   float modifier = (((value * -1) * coefficient) + 1);\n"\
    "\n"\
    "   FragColor = modifier * (texColor-1) + 1;*/\n"\
    "\n"\
    "   float coefficient = texHSL.z * 0.5;\n"\
    "\n"\
    "   FragColor = ((value * coefficient) * texColor) + texColor;\n"\
    "}\n"\
    "";

const char* shadows_fs =
    "#version 330 core\n"\
    "in vec2 TexCoord;\n"\
    "out vec4 FragColor;\n"\
    "uniform sampler2D image;\n"\
    "uniform float value;\n"\
    "\n"\
    "vec4 rgb_to_hsl(vec4 rgb) {\n"\
    "   vec4 hsl = rgb;\n"\
    "\n"\
    "   float max_val = max(rgb.r, max(rgb.g, rgb.b));\n"\
    "   float min_val = min(rgb.r, min(rgb.g, rgb.b));\n"\
    "   float delta = max_val - min_val;\n"\
    "\n"\
    "   if (delta == 0) {\n"\
    "      return vec4(0,0,rgb.r,rgb.a);\n"\
    "   }\n"\
    "   \n"\
    "   if (max_val == rgb.r) {\n"\
    "      hsl.x = 60 * mod(((rgb.g-rgb.b)/delta), 6);\n"\
    "   }\n"\
    "   if (max_val == rgb.g) {\n"\
    "      hsl.x = 60 * ((rgb.b-rgb.r)/delta + 2);\n"\
    "   }\n"\
    "   if (max_val == rgb.b) {\n"\
    "      hsl.x = 60 * ((rgb.r-rgb.g)/delta + 4);\n"\
    "   }\n"\
    "\n"\
    "   if (hsl.x < 0) {\n"\
    "      hsl.x = 360+hsl.x;\n"\
    "   }\n"\
    "\n"\
    "   hsl.z = (max_val + min_val) / 2;\n"\
    "   hsl.y = delta / (1 - abs(2*(hsl.z) - 1));\n"\
    "\n"\
    "   return hsl;\n"\
    "}\n"\
    "\n"\
    "vec4 hsl_to_rgb(vec4 hsl) {\n"\
    "   float c = hsl.y * (1 - abs(2*hsl.z-1));\n"\
    "   float x = c * (1 - abs(mod(hsl.x/60, 2) - 1));\n"\
    "   float m = hsl.z - c/2;\n"\
    "\n"\
    "   float r = 0;\n"\
    "   float g = 0;\n"\
    "   float b = 0;\n"\
    "\n"\
    "   if (hsl.x >= 0 && hsl.x < 60) {\n"\
    "      r = c;\n"\
    "      g = x;\n"\
    "   }\n"\
    "   if (hsl.x >= 60 && hsl.x < 120) {\n"\
    "      r = x;\n"\
    "      g = c;\n"\
    "   }\n"\
    "   if (hsl.x >= 120 && hsl.x < 180) {\n"\
    "      g = c;\n"\
    "      b = x;\n"\
    "   }\n"\
    "   if (hsl.x >= 180 && hsl.x < 240) {\n"\
    "      g = x;\n"\
    "      b = c;\n"\
    "   }\n"\
    "   if (hsl.x >= 240 && hsl.x < 300) {\n"\
    "      r = x;\n"\
    "      b = c;\n"\
    "   }\n"\
    "   if (hsl.x >= 300 && hsl.x < 360) {\n"\
    "      r = c;\n"\
    "      b = x;\n"\
    "   }\n"\
    "\n"\
    "   return vec4(r+m, g+m, b+m, hsl.w);\n"\
    "}\n"\
    "\n"\
    "void main() {\n"\
    "   vec4 texColor = texture(image, TexCoord);\n"\
    "   \n"\
    "   vec4 texHSL = rgb_to_hsl(texColor);\n"\
    "\n"\
    "   /*float coefficient = 1 - texHSL.z * 1.5;\n"\
    "   if (coefficient < 0) {\n"\
    "      coefficient = 0;\n"\
    "   }\n"\
    "\n"\
    "   float modifier = (((value * -1) * coefficient) + 1);\n"\
    "\n"\
    "   FragColor = modifier * (texColor-1) + 1;*/\n"\
    "\n"\
    "   float coefficient = 1 - texHSL.z * 1.5;\n"\
    "   if (coefficient < 0) {\n"\
    "      coefficient = 0;\n"\
    "   }\n"\
    "\n"\
    "   FragColor = ((value * coefficient) * texColor) + texColor;\n"\
    "}\n"\
    "";

const char* temperature_fs =
    "#version 330 core\n"\
    "in vec2 TexCoord;\n"\
    "out vec4 FragColor;\n"\
    "uniform sampler2D image;\n"\
    "uniform float value;\n"\
    "void main() {\n"\
    "   vec4 texColor = texture(image, TexCoord);\n"\
    "   texColor.r += value;\n"\
    "   texColor.b -= value;\n"\
    "   FragColor = texColor;\n"\
    "}\n"\
    "";

const char* color_hue_fs =
    "#version 330 core\n"\
    "in vec2 TexCoord;\n"\
    "out vec4 FragColor;\n"\
    "uniform sampler2D image;\n"\
    "uniform float value;\n"\
    "uniform float hue;\n"\
    "\n"\
    "vec4 rgb_to_hsl(vec4 rgb) {\n"\
    "   vec4 hsl = rgb;\n"\
    "\n"\
    "   float max_val = max(rgb.r, max(rgb.g, rgb.b));\n"\
    "   float min_val = min(rgb.r, min(rgb.g, rgb.b));\n"\
    "   float delta = max_val - min_val;\n"\
    "\n"\
    "   if (delta == 0) {\n"\
    "      return vec4(0,0,rgb.r,rgb.a);\n"\
    "   }\n"\
    "   \n"\
    "   if (max_val == rgb.r) {\n"\
    "      hsl.x = 60 * mod(((rgb.g-rgb.b)/delta), 6);\n"\
    "   }\n"\
    "   if (max_val == rgb.g) {\n"\
    "      hsl.x = 60 * ((rgb.b-rgb.r)/delta + 2);\n"\
    "   }\n"\
    "   if (max_val == rgb.b) {\n"\
    "      hsl.x = 60 * ((rgb.r-rgb.g)/delta + 4);\n"\
    "   }\n"\
    "\n"\
    "   if (hsl.x < 0) {\n"\
    "      hsl.x = 360+hsl.x;\n"\
    "   }\n"\
    "\n"\
    "   hsl.z = (max_val + min_val) / 2;\n"\
    "   hsl.y = delta / (1 - abs(2*(hsl.z) - 1));\n"\
    "\n"\
    "   return hsl;\n"\
    "}\n"\
    "\n"\
    "vec4 hsl_to_rgb(vec4 hsl) {\n"\
    "   float c = hsl.y * (1 - abs(2*hsl.z-1));\n"\
    "   float x = c * (1 - abs(mod(hsl.x/60, 2) - 1));\n"\
    "   float m = hsl.z - c/2;\n"\
    "\n"\
    "   float r = 0;\n"\
    "   float g = 0;\n"\
    "   float b = 0;\n"\
    "\n"\
    "   if (hsl.x >= 0 && hsl.x < 60) {\n"\
    "      r = c;\n"\
    "      g = x;\n"\
    "   }\n"\
    "   if (hsl.x >= 60 && hsl.x < 120) {\n"\
    "      r = x;\n"\
    "      g = c;\n"\
    "   }\n"\
    "   if (hsl.x >= 120 && hsl.x < 180) {\n"\
    "      g = c;\n"\
    "      b = x;\n"\
    "   }\n"\
    "   if (hsl.x >= 180 && hsl.x < 240) {\n"\
    "      g = x;\n"\
    "      b = c;\n"\
    "   }\n"\
    "   if (hsl.x >= 240 && hsl.x < 300) {\n"\
    "      r = x;\n"\
    "      b = c;\n"\
    "   }\n"\
    "   if (hsl.x >= 300 && hsl.x < 360) {\n"\
    "      r = c;\n"\
    "      b = x;\n"\
    "   }\n"\
    "\n"\
    "   return vec4(r+m, g+m, b+m, hsl.w);\n"\
    "}\n"\
    "\n"\
    "void main() {\n"\
    "   vec4 texColor = texture(image, TexCoord);\n"\
    "   \n"\
    "   vec4 texHSL = rgb_to_hsl(texColor);\n"\
    "\n"\
    "   float coeff = max(1 - min(abs(texHSL.x - hue), abs(texHSL.x - 360 - hue)) / 70.0, 0);\n"\
    "\n"\
    "   texHSL.x = texHSL.x + (value * coeff);\n"\
    "\n"\
    "   texHSL.x = abs(mod(texHSL.x, 360)); // if we go over 360 or into negative, normalize it\n"\
    "\n"\
    "   FragColor = hsl_to_rgb(texHSL);\n"\
    "}\n"\
    "";

const char* color_lightness_fs =
    "#version 330 core\n"\
    "in vec2 TexCoord;\n"\
    "out vec4 FragColor;\n"\
    "uniform sampler2D image;\n"\
    "uniform float value;\n"\
    "uniform float hue;\n"\
    "\n"\
    "vec4 rgb_to_hsl(vec4 rgb) {\n"\
    "   vec4 hsl = rgb;\n"\
    "\n"\
    "   float max_val = max(rgb.r, max(rgb.g, rgb.b));\n"\
    "   float min_val = min(rgb.r, min(rgb.g, rgb.b));\n"\
    "   float delta = max_val - min_val;\n"\
    "\n"\
    "   if (delta == 0) {\n"\
    "      return vec4(0,0,rgb.r,rgb.a);\n"\
    "   }\n"\
    "   \n"\
    "   if (max_val == rgb.r) {\n"\
    "      hsl.x = 60 * mod(((rgb.g-rgb.b)/delta), 6);\n"\
    "   }\n"\
    "   if (max_val == rgb.g) {\n"\
    "      hsl.x = 60 * ((rgb.b-rgb.r)/delta + 2);\n"\
    "   }\n"\
    "   if (max_val == rgb.b) {\n"\
    "      hsl.x = 60 * ((rgb.r-rgb.g)/delta + 4);\n"\
    "   }\n"\
    "\n"\
    "   if (hsl.x < 0) {\n"\
    "      hsl.x = 360+hsl.x;\n"\
    "   }\n"\
    "\n"\
    "   hsl.z = (max_val + min_val) / 2;\n"\
    "   hsl.y = delta / (1 - abs(2*(hsl.z) - 1));\n"\
    "\n"\
    "   return hsl;\n"\
    "}\n"\
    "\n"\
    "vec4 hsl_to_rgb(vec4 hsl) {\n"\
    "   float c = hsl.y * (1 - abs(2*hsl.z-1));\n"\
    "   float x = c * (1 - abs(mod(hsl.x/60, 2) - 1));\n"\
    "   float m = hsl.z - c/2;\n"\
    "\n"\
    "   float r = 0;\n"\
    "   float g = 0;\n"\
    "   float b = 0;\n"\
    "\n"\
    "   if (hsl.x >= 0 && hsl.x < 60) {\n"\
    "      r = c;\n"\
    "      g = x;\n"\
    "   }\n"\
    "   if (hsl.x >= 60 && hsl.x < 120) {\n"\
    "      r = x;\n"\
    "      g = c;\n"\
    "   }\n"\
    "   if (hsl.x >= 120 && hsl.x < 180) {\n"\
    "      g = c;\n"\
    "      b = x;\n"\
    "   }\n"\
    "   if (hsl.x >= 180 && hsl.x < 240) {\n"\
    "      g = x;\n"\
    "      b = c;\n"\
    "   }\n"\
    "   if (hsl.x >= 240 && hsl.x < 300) {\n"\
    "      r = x;\n"\
    "      b = c;\n"\
    "   }\n"\
    "   if (hsl.x >= 300 && hsl.x < 360) {\n"\
    "      r = c;\n"\
    "      b = x;\n"\
    "   }\n"\
    "\n"\
    "   return vec4(r+m, g+m, b+m, hsl.w);\n"\
    "}\n"\
    "\n"\
    "void main() {\n"\
    "   vec4 texColor = texture(image, TexCoord);\n"\
    "   \n"\
    "   vec4 texHSL = rgb_to_hsl(texColor);\n"\
    "\n"\
    "   float coeff = max(1 - min(abs(texHSL.x - hue), abs(texHSL.x - 360 - hue)) / 70.0, 0);\n"\
    "\n"\
    "   texHSL.z = texHSL.z * ((value * coeff * (texHSL.y)) + 1);\n"\
    "\n"\
    "   FragColor = hsl_to_rgb(texHSL);\n"\
    "}\n"\
    "";

const char* color_saturation_fs =
    "#version 330 core\n"\
    "in vec2 TexCoord;\n"\
    "out vec4 FragColor;\n"\
    "uniform sampler2D image;\n"\
    "uniform float value;\n"\
    "uniform float hue;\n"\
    "\n"\
    "vec4 rgb_to_hsl(vec4 rgb) {\n"\
    "   vec4 hsl = rgb;\n"\
    "\n"\
    "   float max_val = max(rgb.r, max(rgb.g, rgb.b));\n"\
    "   float min_val = min(rgb.r, min(rgb.g, rgb.b));\n"\
    "   float delta = max_val - min_val;\n"\
    "\n"\
    "   if (delta == 0) {\n"\
    "      return vec4(0,0,rgb.r,rgb.a);\n"\
    "   }\n"\
    "   \n"\
    "   if (max_val == rgb.r) {\n"\
    "      hsl.x = 60 * mod(((rgb.g-rgb.b)/delta), 6);\n"\
    "   }\n"\
    "   if (max_val == rgb.g) {\n"\
    "      hsl.x = 60 * ((rgb.b-rgb.r)/delta + 2);\n"\
    "   }\n"\
    "   if (max_val == rgb.b) {\n"\
    "      hsl.x = 60 * ((rgb.r-rgb.g)/delta + 4);\n"\
    "   }\n"\
    "\n"\
    "   if (hsl.x < 0) {\n"\
    "      hsl.x = 360+hsl.x;\n"\
    "   }\n"\
    "\n"\
    "   hsl.z = (max_val + min_val) / 2;\n"\
    "   hsl.y = delta / (1 - abs(2*(hsl.z) - 1));\n"\
    "\n"\
    "   return hsl;\n"\
    "}\n"\
    "\n"\
    "vec4 hsl_to_rgb(vec4 hsl) {\n"\
    "   float c = hsl.y * (1 - abs(2*hsl.z-1));\n"\
    "   float x = c * (1 - abs(mod(hsl.x/60, 2) - 1));\n"\
    "   float m = hsl.z - c/2;\n"\
    "\n"\
    "   float r = 0;\n"\
    "   float g = 0;\n"\
    "   float b = 0;\n"\
    "\n"\
    "   if (hsl.x >= 0 && hsl.x < 60) {\n"\
    "      r = c;\n"\
    "      g = x;\n"\
    "   }\n"\
    "   if (hsl.x >= 60 && hsl.x < 120) {\n"\
    "      r = x;\n"\
    "      g = c;\n"\
    "   }\n"\
    "   if (hsl.x >= 120 && hsl.x < 180) {\n"\
    "      g = c;\n"\
    "      b = x;\n"\
    "   }\n"\
    "   if (hsl.x >= 180 && hsl.x < 240) {\n"\
    "      g = x;\n"\
    "      b = c;\n"\
    "   }\n"\
    "   if (hsl.x >= 240 && hsl.x < 300) {\n"\
    "      r = x;\n"\
    "      b = c;\n"\
    "   }\n"\
    "   if (hsl.x >= 300 && hsl.x < 360) {\n"\
    "      r = c;\n"\
    "      b = x;\n"\
    "   }\n"\
    "\n"\
    "   return vec4(r+m, g+m, b+m, hsl.w);\n"\
    "}\n"\
    "\n"\
    "void main() {\n"\
    "   vec4 texColor = texture(image, TexCoord);\n"\
    "   \n"\
    "   vec4 texHSL = rgb_to_hsl(texColor);\n"\
    "\n"\
    "   float coeff = max(1 - min(abs(texHSL.x - hue), abs(texHSL.x - 360 - hue)) / 70.0, 0);\n"\
    "\n"\
    "   texHSL.y = texHSL.y * ((value * coeff) + 1);\n"\
    "\n"\
    "   FragColor = hsl_to_rgb(texHSL);\n"\
    "}\n"\
    "";

const char* smart_denoise_fs =
    "//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n"\
    "//  Copyright (c) 2018-2019 Michele Morrone\n"\
    "//  All rights reserved.\n"\
    "//\n"\
    "//  https://michelemorrone.eu - https://BrutPitt.com\n"\
    "//\n"\
    "//  me@michelemorrone.eu - brutpitt@gmail.com\n"\
    "//  twitter: @BrutPitt - github: BrutPitt\n"\
    "//  \n"\
    "//  https://github.com/BrutPitt/glslSmartDeNoise/\n"\
    "//\n"\
    "//  This software is distributed under the terms of the BSD 2-Clause license\n"\
    "//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n"\
    "#version 330 core\n"\
    "\n"\
    "#define INV_SQRT_OF_2PI 0.39894228040143267793994605993439  // 1.0/SQRT_OF_2PI\n"\
    "#define INV_PI 0.31830988618379067153776752674503\n"\
    "\n"\
    "in vec2 TexCoord;\n"\
    "out vec4 FragColor;\n"\
    "uniform sampler2D image;\n"\
    "uniform float sigma;\n"\
    "uniform float kSigma;\n"\
    "uniform float threshold;\n"\
    "\n"\
    "//  Parameters:\n"\
    "//      sampler2D tex     - sampler image / texture\n"\
    "//      vec2 uv           - actual fragment coord\n"\
    "//      float sigma  >  0 - sigma Standard Deviation\n"\
    "//      float kSigma >= 0 - sigma coefficient\n"\
    "//          kSigma * sigma  -->  radius of the circular kernel\n"\
    "//      float threshold   - edge sharpening threshold\n"\
    "vec4 smartDeNoise(sampler2D tex, vec2 uv, float sigma, float kSigma, float threshold)\n"\
    "{\n"\
    "    float radius = round(kSigma*sigma);\n"\
    "    float radQ = radius * radius;\n"\
    "\n"\
    "    float invSigmaQx2 = .5 / (sigma * sigma);      // 1.0 / (sigma^2 * 2.0)\n"\
    "    float invSigmaQx2PI = INV_PI * invSigmaQx2;    // // 1/(2 * PI * sigma^2)\n"\
    "\n"\
    "    float invThresholdSqx2 = .5 / (threshold * threshold);     // 1.0 / (sigma^2 * 2.0)\n"\
    "    float invThresholdSqrt2PI = INV_SQRT_OF_2PI / threshold;   // 1.0 / (sqrt(2*PI) * sigma)\n"\
    "\n"\
    "    vec4 centrPx = texture(tex,uv);\n"\
    "\n"\
    "    float zBuff = 0.0;\n"\
    "    vec4 aBuff = vec4(0.0);\n"\
    "    vec2 size = vec2(textureSize(tex, 0));\n"\
    "\n"\
    "    vec2 d;\n"\
    "    for (d.x=-radius; d.x <= radius; d.x++) {\n"\
    "        float pt = sqrt(radQ-d.x*d.x);       // pt = yRadius: have circular trend\n"\
    "        for (d.y=-pt; d.y <= pt; d.y++) {\n"\
    "            float blurFactor = exp( -dot(d , d) * invSigmaQx2 ) * invSigmaQx2PI;\n"\
    "\n"\
    "            vec4 walkPx =  texture(tex,uv+d/size);\n"\
    "            \n"\
    "            vec4 dC = walkPx-centrPx;\n"\
    "            float deltaFactor = exp( -dot(dC.rgb, dC.rgb) * invThresholdSqx2) * invThresholdSqrt2PI * blurFactor;\n"\
    "\n"\
    "            zBuff += deltaFactor;\n"\
    "            aBuff += deltaFactor*walkPx;\n"\
    "        }\n"\
    "    }\n"\
    "    return aBuff/zBuff;\n"\
    "}\n"\
    "\n"\
    "\n"\
    "void main() {\n"\
    "   FragColor = smartDeNoise(image, TexCoord, sigma, kSigma, threshold);\n"\
    "}\n"\
    "";

